-- get the addon namespace
local addon, ns = ...
-- get the config values
local cfg = ns.cfg
-- holder for some lib functions
local lib = _G["HEDD_lib"] or CreateFrame("Frame","HEDD_lib")
ns.lib = lib

lib.UpdateCast = function(spellID)
	cfg.Casting.name, _, _, _, cfg.Casting.caststart, cfg.Casting.castend, _, cfg.Casting.castid = UnitCastingInfo("player")
	if cfg.Casting.name then
		cfg.Casting.iscasting=true
		cfg.Casting.caststart=cfg.Casting.caststart / 1e3
		cfg.Casting.castend=cfg.Casting.castend / 1e3
		cfg.Casting.id=spellID or lib.GetSpellIdbyName(cfg.Casting.name)
		cfg.Casting.spell=cfg.id2spell[cfg.Casting.id] or "unknown"
		cfg.Casting.cost=lib.GetSpellCost(cfg.Casting.spell)
		
		return true
	else
		cfg.Casting.iscasting=false
		cfg.Casting.cost=0
		cfg.Casting.id=false
		cfg.Casting.castid=false
		cfg.Casting.spell=false
		return false
	end
end

lib.IsCasting = function()
	if cfg.Casting.iscasting and cfg.Casting.castend>GetTime() then
		return true
	else
		return false
	end
end

lib.GetCastingSpell=function()
	if lib.IsCasting() then
		return cfg.Casting.spell
	end
	return nil
end

lib.GetCastingSpellCost=function()
	if lib.IsCasting() then
		return cfg.Casting.cost
	end
	return 0
end

lib.SpellCastingLeft=function(spell)
	if lib.IsCasting() then
		if (not spell) or (spell and cfg.spells[spell] and cfg.spells[spell].name==cfg.Casting.name) then
			return math.max(0,(cfg.Casting.castend-GetTime()))
		end
	end
	return 0
end

lib.SpellCasting=function(spell)
	if lib.IsCasting() then
		if (not spell) or (spell and cfg.spells[spell] and cfg.spells[spell].id==cfg.Casting.id) then
			return true
		else
			return false
		end
	end
	return false
end

lib.SaveSpell = function(spellID)
	cfg.spellcast[spellID]=GetTime()
	cfg.lastspell2=cfg.lastspell
	cfg.lastspell=spellID
	cfg.lastspell_time=GetTime()
end

lib.IsLastSpell = function(spell)
	if cfg.lastspell and cfg.spells[spell] and cfg.lastspell==cfg.spells[spell].id then
		return true
	end
	return nil
end

lib.GetLastSpell = function (spells)
	for _,cspell in ipairs(spells) do
		if cfg.spells[cspell] and cfg.spellcast[cfg.spells[cspell].id] then
			if GetTime()<(cfg.spellcast[cfg.spells[cspell].id]-cfg.spells[cspell].castingTime+1) then
				return true
			end
		end
	end
	return nil
end

lib.SetInterrupt = function(spell,ids,nousecheck,powerCost_real)
	lib.AddSpell(spell,ids,nousecheck,powerCost_real)
	if cfg.spells[spell] then
		cfg.Interrupt.spell=spell
		cfg.case[spell] = function()
			if cfg.Interrupt.spell and lib.IsCastingInterruptible() and lib.GetTargetCastLeft()>lib.GetSpellCD(spell) then
				return lib.SimpleCDCheck(spell)
			end
			return nil
		end
	end
end

lib.IsCastingInterruptible = function()
	if cfg.Interrupt.iscasting and cfg.Interrupt.castend>GetTime() and cfg.Interrupt.notInterruptible==false then
		return true
	else
		return false
	end
end

lib.GetTargetCastLeft = function()
	return math.max(0,cfg.Interrupt.castend-GetTime())
end

local interrupt_old={}
lib.UpdateTargetCast = function(spellID)
	spellID=spellID or false
	hedlib.shallowCopy(cfg.Interrupt,interrupt_old)
	cfg.Interrupt.name, _, _, _, cfg.Interrupt.caststart, cfg.Interrupt.castend, _, cfg.Interrupt.castid, cfg.Interrupt.notInterruptible = UnitCastingInfo("target")
	if cfg.Interrupt.name then
		--print(tostring(cfg.Interrupt.notInterruptible))
		cfg.Interrupt.iscasting=true
		cfg.Interrupt.caststart=cfg.Interrupt.caststart / 1e3
		cfg.Interrupt.castend=cfg.Interrupt.castend / 1e3
		cfg.Interrupt.id=spellID
	else
		cfg.Interrupt.iscasting=false
		cfg.Interrupt.id=false
		cfg.Interrupt.castid=false
	end
	if not hedlib.ArrayNotChanged(cfg.Interrupt,interrupt_old) then
		cfg.Update=true
	end
end
